<?php
/*
Plugin Name: Backup
Version: 1.1.6
Plugin URI: http://hel.io/wordpress/backup/
Description: Backup your WordPress website to Google Drive.
Author: Sorin Iclanzan
Author URI: http://hel.io/
*/

// This is run when you activate the plugin, checking for compatibility, adding the default options to the database
register_activation_hook(__FILE__,'backup_activation');
function backup_activation() {
	// Check for compatibility
    try {
        // try to create backup folder inside wp-contents
        if ( !is_dir( WP_CONTENT_DIR . '/backup' ) )
            if ( ! @mkdir( WP_CONTENT_DIR . '/backup', 0755 ) )
                throw new Exception(__('Could not create \'backup\' folder inside \'wp-content\'. Either create it yourself or set the right permissions and try activating the plugin again.', 'backup'));
        // try to create log file
        if ( FALSE === file_put_contents( WP_CONTENT_DIR . '/backup/backup.log', "#Fields:\tdate\ttime\ttype\tmessage\tfile\tline\n" ) )
            throw new Exception(__('Could not create log file. Make sure the web server has the right permissions to write to the \'backup\' folder.', 'backup'));

        // check allow_url_fopen
        if(!ini_get('allow_url_fopen')) {
            throw new Exception(__('Please enable \'allow_url_fopen\' in PHP. Backup can not function without it.', 'backup'));
        }

        // check zip extension
        if(!extension_loaded('zip')) {
            throw new Exception(__('Please load the \'zip\' PHP extension. It is needed in order to create the archive to be backed up.', 'backup'));
        }

        // check file_get_contents to docs.google.com
        $context = array(
            'http' => array(
                'timeout'  => 3,
            )
        );
        $result = @file_get_contents('http://docs.google.com/', false, stream_context_create($context));
        if($result === false) {
            throw new Exception(__('Cannot connect to Google Drive. Backup could not be activated.', 'backup'));
        }

        // check OpenSSL
        if(!function_exists('openssl_open')) {
            throw new Exception(__('Please enable OpenSSL in PHP. Backup needs it to communicate with Google Drive.', 'backup'));
        }

        // check SimpleXMLElement
        if(!class_exists('SimpleXMLElement')) {
            throw new Exception(__('Please enable SimpleXMLElement in PHP. Backup could not be activated.', 'backup'));
        }
    }
    catch(Exception $e) {
        $plugin_basename = dirname(plugin_basename(__FILE__));
        deactivate_plugins($plugin_basename.'/backup.php', true);
        echo '<div id="message" class="error">' . $e->getMessage() . '</div>';
        trigger_error('Could not activate Backup.', E_USER_ERROR);
    }

	// Default options
	$backup_options = array(
        'token' => '',
        'folder' => '',
        'frequency' => 'never',
        'client_id' => '',
        'client_secret' => '',
        'last_backup' => '',
        'local_number' => 10,
        'drive_number' => 10,
        'local_files' => array(),
        'drive_files' => array(),
    );
	
	// Add options
	update_option( 'backup_options', $backup_options );
	
}

/**
 * Backup Deactivation
 *
 * This function is called whenever the plugin is being deactivated and removes
 * all files and directories it created as well as the options stored in the database.
 * It also revokes access to the Google Account associated with it and removes all scheduled events created.
 */
function backup_deactivation() {
    $options = get_option( 'backup_options' );
    if ( ! empty( $options['token'] ) )
        @file_get_contents( 'https://accounts.google.com/o/oauth2/revoke?token=' . $options['token'] );
    delete_option( 'backup_options' );
    if ( wp_next_scheduled( 'backup_schedule' ) ) {
        wp_clear_scheduled_hook('backup_schedule');
    }
    rrmdir( WP_CONTENT_DIR . '/backup' );
    if ( file_exists( WP_CONTENT_DIR . '/backup.sql' ) )
        unlink( WP_CONTENT_DIR . '/backup.sql' );
}
register_deactivation_hook( __FILE__, 'backup_deactivation' );

// Add custom schedule intervals
add_filter( 'cron_schedules', 'cron_add_intervals' );
function cron_add_intervals( $schedules ) {
    $schedules['weekly'] = array(
        'interval' => 604800,
        'display' => __( 'Weekly', 'backup' )
    );
    $schedules['monthly'] = array(
        'interval' => 2592000,
        'display' => __( 'Monthly', 'backup' )
    );
    return $schedules;
}

// Add options page in the admin menu
add_action('admin_menu','backup_menu');
function backup_menu() {
	add_options_page('Backup Settings', 'Backup', 'manage_options', 'backup', 'backup_options_page');
}

// Add "Settings" link to the plugins page
add_filter( 'plugin_action_links', 'backup_action_links',10,2);
function backup_action_links( $links, $file ) {
	if ( $file != plugin_basename( __FILE__ ))
		return $links;

	$settings_link = '<a href="options-general.php?page=backup">Settings</a>';

	array_unshift( $links, $settings_link );

	return $links;
}

// Display options page
function backup_options_page() {
    // Display messages and errors
    if ( isset( $_GET['message'] ) )
        echo '<div id="message" class="updated fade">
<p>' . $_GET['message'] . '</p>
</div>';
    if ( isset( $_GET['error'] ) )
        echo '<div id="message" class="error">
<p>' . $_GET['error'] . '</p>
</div>';
    
    // Check to see if we have authorization; if we have a token saved it means we have authorization.
    $options = get_option('backup_options');
    ?>
    <div class="wrap">
        <div id="icon-options-general" class="icon32"><br/></div><h2><?php _e( 'Backup Settings', 'backup' ); ?></h2>
        <div id="poststuff" class="metabox-holder has-right-sidebar">
            <div class="inner-sidebar">
                <div id="submitdiv" class="postbox">
                    <h3><?php _e( 'Status', 'backup' ); ?></h3>
                    <div class="inside">
                        <div class="misc-pub-section"><?php _e( 'Most recent backup:', 'backup' ); ?><br/><?php 
                            if ( $options['last_backup'] ) {
                                echo '<strong>' . date( __( "M j, Y \a\\t H:i:s", 'backup' ), $options['last_backup'] ) . '</strong>';
                                if ( $options['local_number'] > 0 ) {
                                    ?><br/><br/><a class="button-secondary" title="<?php _e( 'Download most recent backup', 'backup'); ?>" href="<?php echo WP_CONTENT_URL . substr( end( $options['local_files'] ), strlen( WP_CONTENT_DIR ) ); ?>">Download backup</a><?php
                                }
                            }
                            else {
                                echo '<strong>' . __( 'never', 'backup' ) . '</strong>';
                            }    
                                ?></div>
                        <div class="misc-pub-section"><?php _e( 'Next scheduled backup:', 'backup' ); ?><br/><strong><?php
                            if ( $next = wp_next_scheduled( 'backup_schedule' ) )
                                echo date( __( "M j, Y \a\\t H:i:s", 'backup' ), $next );
                            else
                                _e( 'never', 'backup' );
                         ?></strong></div>
                         <div class="misc-pub-section misc-pub-section-last"><?php _e( 'Manual backup URL:', 'backup' ); ?><br/><kbd><?php echo home_url( '/backup/' ); ?></kbd></div>
                    </div>
                </div>
            </div>
        </div>       
        <h3><?php _e( 'Authorization', 'backup' ); ?></h3>
        <?php if ( $options['token'] == '' ) { ?>
        <form action="options-general.php?page=backup&action=auth" method="post">
            <p><?php _e( 'Before we can do anything you need to give this plugin authorization to use Google Dirve.', 'backup' ); ?></p>
            <p><?php printf( __( 'You can create a Client ID in the API Access section of your %s if you don\'t have one. A client secret will also be generated for you as long as you select \'Web Application\' as the application type.', 'backup' ), '<a href="https://code.google.com/apis/console/" target="_blank">Google API Console</a>' ); ?></p>
            <p><?php printf( __( 'Make sure to add %s as the authorized redirect URI when asked.', 'backup' ), '<kbd>' . admin_url( 'options-general.php?page=backup&action=auth' ) . '</kbd>' ); ?></p>
            <table class="form-table">
                <tbody>
                    <tr valign="top">
                        <th scope="row"><?php _e( 'Client ID', 'backup' ); ?></th>
                        <td><input name='client_id' type='text' class='regular-text' value='<?php echo $options['client_id']; ?>' /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><?php _e( 'Client secret', 'backup' ); ?></th>
                        <td><input name='client_secret' type='text' class='regular-text' value='<?php echo $options['client_secret']; ?>' /></td>
                    </tr>
                </tbody>
            </table>
            <p class="submit">
                <input name="authorize" type="submit" class="button-secondary" value="<?php _e('Authorize', 'backup') ?>" />
            </p>
        </form>
        <?php } else { ?>
        <p><?php _e( 'Authorization to use Google Drive has been granted. You can revoke it at any time by clicking the button below.', 'backup' ); ?></p>
        <p class="submit">
            <a href="options-general.php?page=backup&action=auth&state=revoke" class="button-secondary"><?php _e( 'Revoke Authorization', 'backup' ); ?></a>
        </p>
        <?php } ?>

        <form action="options-general.php?page=backup&action=update" method="post">
            <h3><?php _e( 'Settings', 'backup' ); ?></h3>
            <p><?php _e( 'Please enter the name of the Google Drive folder where you want to store backups and select the frequency at which to perform backups.', 'backup' ); ?></p>
            <p><?php _e( 'To get a folder\'s ID navigate to that folder in Google Drive and copy the ID from your browser\'s address bar. It is the part that comes after <kbd>#folders/</kbd>.', 'backup' ); ?> </p>
            <table class="form-table">
                <tbody>
                    <tr valign="top">
                        <th scope="row"><?php _e( 'Backup folder ID', 'backup' ); ?></th>
                        <td>
                            <input name='folder' type='text' class='regular-text' value='<?php echo $options['folder']; ?>' />
                            <span class="description"><?php _e( 'Leave empty to save in root folder.', 'backup' ) ?></span>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><?php _e( 'Store a maximum of', 'backup' ); ?></th>
                        <td>
                            <input name='local_number' type='text' class='small-text' value='<?php echo $options['local_number']; ?>' /> <?php _e( 'backups locally and ', 'backup' ); ?>
                            <input name='drive_number' type='text' class='small-text' value='<?php echo $options['drive_number']; ?>' /> <?php _e( 'backups on Google Drive.', 'backup' ); ?>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row"><?php _e( 'When to backup', 'backup' ); ?></th>
                        <td> 
                            <select name='frequency'>
                                <option value='never' <?php selected( 'never', $options['frequency'] ); ?> ><?php _e( 'Never', 'backup' ); ?></option>
                                <option value='daily' <?php selected( 'daily', $options['frequency'] ); ?> ><?php _e( 'Daily', 'backup' ); ?></option>
                                <option value='weekly' <?php selected( 'weekly', $options['frequency'] ); ?> ><?php _e( 'Weekly', 'backup' ); ?></option>
                                <option value='monthly' <?php selected( 'monthly', $options['frequency'] ); ?> ><?php _e( 'Monthly', 'backup' ); ?></option>
                            </select>
                            <span class="description"><?php _e( "Select <kbd>never</kbd> if you want to add a cron job to do backups.", 'backup' ) ?></span>
                        </td>
                </tbody>
            </table>    
            <p class="submit">
                <input name="submit" type="submit" class="button-primary" value="<?php _e('Save Changes', 'backup') ?>" />
            </p>
        </form>
        
    </div>
    <?php
}

add_action('init', 'backup_update');
function backup_update() {
    if ( is_admin() && isset( $_GET['page'] ) && $_GET['page'] == 'backup' && isset( $_GET['action'] ) && $_GET['action'] == 'update' ) {
        
        $options = get_option('backup_options' );

        // If we dont have a valid recurrence frequency stop function execution
        if ( !in_array($_POST['frequency'], array_keys( wp_get_schedules() ) )  && $_POST['frequency'] != 'never' )
            return false;

        // If number isn't an integer stop function execution
        $local_number = intval( $_POST['local_number'] );
        $drive_number = intval( $_POST['drive_number'] );
        if ( $local_number < 0 || $drive_number < 0 )
            return false;
        
        $options['folder'] = $_POST['folder'];
        $options['local_number'] = $local_number;
        $options['drive_number'] = $drive_number;

        if ( $options['frequency'] != $_POST['frequency'] ) {
            // If we have already scheduled a backup before, clear it first
            if ( wp_next_scheduled( 'backup_schedule' ) ) {
                wp_clear_scheduled_hook('backup_schedule');
            }

            // Schedule backup if frequency is something else than never
            if ( $_POST['frequency'] != 'never' ) {
                wp_schedule_event( current_time( 'timestamp' ), $_POST['frequency'], 'backup_schedule');
            }
        }    

        $options['frequency'] = $_POST['frequency'];
        update_option( 'backup_options', $options );  
    }
}

/**
 * Function to initiate backup if the appropriate page is requested.
 * It hooks to 'template_redirect'.
 */
function manual_backup() {
    if ( $_SERVER['REQUEST_URI'] == '/backup/' || $_SERVER['REQUEST_URI'] == '/backup' ) {
        do_backup();
        header("HTTP/1.1 200 OK");
        exit;
    }
}
add_action('template_redirect', 'manual_backup');

add_action( 'backup_schedule', 'do_backup' );
function do_backup() {
    $options = get_option( 'backup_options' );
    $backup_time = time();
    $file_name = $backup_time . '.zip';
    $file_path = WP_CONTENT_DIR . '/backup/' . $file_name;
    
    // Create database dump sql file
    db_dump();

    // Create archive from wp-content
    backup_log( 'NOTICE', 'Attempting to create archive ' . $file_path );
    $timer_start = microtime( true );
    $result = zip( WP_CONTENT_DIR . '/', $file_path, WP_CONTENT_DIR . '/backup' );

    if ( !$result ) {
        backup_log( 'ERROR', "There was a problem creating the zip file!", __FILE__, __LINE__ );
    }
    else {
        backup_log( 'NOTICE', 'Archive created successfully in ' . ( microtime( true ) - $timer_start ) . ' seconds.' );
        $options['last_backup'] = $backup_time;
        $options['local_files'][] = $file_path;
        if ( $options['drive_number'] > 0 && $options['token'] && $options['client_id'] && $options['client_secret'] ) {
            if ( $access = access_token( $options['token'], $options['client_id'], $options['client_secret'] ) ) {
                backup_log( 'NOTICE', 'Attempting to upload archive to Google Drive' );
                $timer_start = microtime( true );
                if ( ! $id = upload_file( $file_path, $file_name, $options['folder'], $access ) )
                    backup_log( 'ERROR', 'Failed to upload archive to Google Drive!' );
                else {
                    backup_log( 'NOTICE', 'Archive ' . $file_name . ' uploaded to Google Drive in ' . ( microtime( true ) - $timer_start ) . ' seconds' );
                    $options['drive_files'][] = $id;
                }
            }
            else {
                backup_log( 'ERROR', 'Did not receive an access token from Google!', __FILE__, __LINE__ );
            }    
        }
        
        // Delete excess Drive files
        while ( count( $options['drive_files'] ) > $options['drive_number'] )
            if ( delete_file( $r = array_shift( $options['drive_files'] ), $access ) )
                backup_log( 'NOTICE', 'Deleted Google Drive file ' . $r );
            else    
                backup_log( 'WARNING', 'Could not delete Google Drive file ' . $r );
        
        // Delete excess local files    
        while ( count( $options['local_files'] ) > $options['local_number'] )
            if ( unlink( $f = array_shift( $options['local_files'] ) ) )
                backup_log( 'NOTICE', 'Deleted old backup file ' . $f );
            else    
                backup_log( 'WARNING', 'Could not delete file ' . $f );
        
        update_option( 'backup_options', $options );
    }
}

/**
 * Function to upload a file to Google Drive
 *
 * @param  string  $file   Path to the file that is to be uploaded
 * @param  string  $title  Title to be given to the file
 * @param  string  $parent ID of the folder in which to upload the file
 * @param  string  $token  Access token from Google Account
 * @return boolean         Returns TRUE on success, FALSE on failure
 */
function upload_file( $file, $title, $parent = '', $token) {

    $size = filesize( $file );

    $content = '<?xml version=\'1.0\' encoding=\'UTF-8\'?>
<entry xmlns="http://www.w3.org/2005/Atom" xmlns:docs="http://schemas.google.com/docs/2007">
  <category scheme="http://schemas.google.com/g/2005#kind" term="http://schemas.google.com/docs/2007#file"/>
  <title>' . $title . '</title>
</entry>';
    
    $header = array(
        'Authorization: Bearer ' . $token,
        'Content-Length: ' . strlen( $content ),
        'Content-Type: application/atom+xml',
        'X-Upload-Content-Type: application/zip',
        'X-Upload-Content-Length: ' . $size,
        'GData-Version: 3.0'
    );

    $context = array(
        'http' => array(
            'ignore_errors' => true,
            'follow_location' => false,
            'method'  => 'POST',
            'header'  => join( "\r\n", $header ),
            'content' => $content
        )
    );

    $url = get_resumable_create_media_link( $token, $parent );
    if ( $url )
        $url .= '?convert=false'; // needed to upload a file
    else {
        backup_log( 'ERROR', 'Could not retrieve resumable create media link.', __FILE__, __LINE__ );
        return false;
    }
    
    $result = @file_get_contents( $url, false, stream_context_create( $context ) );
    if ( $result !== FALSE ) {
        if ( strpos( $response = array_shift( $http_response_header ), '200' ) ) {
            $response_header = array();
            foreach ( $http_response_header as $header_line ) {
                list( $key, $value ) = explode( ':', $header_line, 2 );
                $response_header[trim( $key )] = trim( $value );
            }
            if ( isset( $response_header['Location'] ) ) {
                $next_location = $response_header['Location'];
                $pointer = 0;
                $max_chunk_size = 524288;
                while ( $pointer < $size - 1 ) {
                    $chunk = file_get_contents( $file, false, NULL, $pointer, $max_chunk_size );
                    $next_location = upload_chunk( $next_location, $chunk, $pointer, $size, $token );
                    if( $next_location === false ) {
                        return false;
                    }    
                    // if object it means we have our simpleXMLElement response
                    if ( is_object( $next_location ) ) {
                        // return resource Id
                        return substr( $next_location->children( "http://schemas.google.com/g/2005" )->resourceId, 5 );
                    }
                    $pointer += strlen( $chunk );
                    
                }
            } 
        }
        else {
            backup_log( 'ERROR', 'Bad response: ' . $response . ' Response header: ' . var_export( $response_header, true ) . ' Response body: ' . $result . ' Request URL: ' . $url, __FILE__, __LINE__ );
            return false;
        }
    }
    else {
        backup_log( 'ERROR', 'Unable to request file from ' . $url, __FILE__, __LINE__ );
    }    
}
/**
 * Handles the upload to Google Drive of a single chunk of a file
 *
 * @param  string  $location URL where the chunk needs to be uploaded
 * @param  string  $chunk    Part of the file to upload
 * @param  integer $pointer  The byte number marking the beginning of the chunk in file
 * @param  integer $size     The size of the file the chunk is part of, in bytes
 * @param  string  $token    Google Account access token
 * @return string|boolean    The funcion returns the location where the next chunk needs to be uploaded, TRUE if the last chunk was uploaded or FALSE on failure
 */
function upload_chunk( $location, $chunk, $pointer, $size, $token ) {
    $chunk_size = strlen( $chunk );
    $bytes = (string)$pointer . '-' . (string)($pointer + $chunk_size - 1) . '/' . (string)$size;
    $header = array(
        'Authorization: Bearer ' . $token,
        'Content-Length: ' . $chunk_size,
        'Content-Type: application/zip',
        'Content-Range: bytes ' . $bytes,
        'GData-Version: 3.0'
    );
    $context = array(
        'http' => array(
            'ignore_errors' => true,
            'follow_location' => false,
            'method' => 'PUT',
            'header' => join( "\r\n", $header ),
            'content' => $chunk
        )
    );

    $result = @file_get_contents( $location, false, stream_context_create( $context ) );

    if ( isset( $http_response_header ) ) {
        $response = array_shift( $http_response_header );
        $headers = array();
        foreach ( $http_response_header as $header_line ) {
            list( $key, $value ) = explode( ':', $header_line, 2 );
            $headers[trim( $key )] = trim( $value );
        }
        
        if ( strpos( $response, '308' ) ) {
            if ( isset( $headers['Location'] ) ) {
                return $headers['Location'];
            }
            else 
                return $location;
        }
        elseif ( strpos( $response, '201' ) ) {
            $xml = simplexml_load_string( $result );
            if ( $xml === false ) {
                backup_log( 'ERROR', 'Could not create SimpleXMLElement from ' . $result, __FILE__, __LINE__ );
                return false;
            }
            else
                return $xml;
        }
        else {
            backup_log( 'ERROR', 'Bad response: ' . $response, __FILE__, __LINE__ );
            return false;
        }
    }
    else {
        backup_log( 'ERROR', 'Received no response from ' . $location . ' while trying to upload bytes ' . $bytes );
        return false;
    }
}

/**
 * Deletes a file from Google Drive
 *
 * @param  string $id    Gdata resource Id of the file to be deleted
 * @param  string $token Google Account access token
 * @return boolean       Returns TRUE on success, FALSE on failure
 */
function delete_file( $id, $token ) {
    $header = array(
        'If-Match: *',
        'Authorization: Bearer ' . $token,
        'GData-Version: 3.0'
    );
    $context = array(
        'http' => array(
            'method' => 'DELETE',
            'header' => join( "\r\n", $header )
        )
    );
    stream_context_set_default( $context );
    $headers = get_headers( 'https://docs.google.com/feeds/default/private/full/' . $id . '?delete=true',1 );

    if ( strpos( $headers[0], '200' ) )
        return true;
    return false;
}
/**
 * Get the resumable-create-media link needed to upload files
 *
 * @param  string $token  The Google Account access token
 * @param  string $parent The Id of the folder where the upload is to be made. Default is empty string.
 * @return string|boolean Returns a link on success, FALSE on failure. 
 */
function get_resumable_create_media_link( $token, $parent = '' ) {
    $header = array(
        'Authorization: Bearer ' . $token,
        'GData-Version: 3.0'
    );
    $context = array(
        'http' => array(
            'ignore_errors' => true,
            'method' => 'GET',
            'header' => join( "\r\n", $header )
        )
    );
    $url = 'https://docs.google.com/feeds/default/private/full';

    if ( $parent )
        $url .= '/' . $parent;

    $result = @file_get_contents( $url, false, stream_context_create( $context ) );

    if ( $result !== false ) {
        $xml = simplexml_load_string( $result );
        if ( $xml === false ) {
            backup_log( 'ERROR', 'Could not create SimpleXMLElement from ' . $result, __FILE__, __LINE__ );
                return false;
        }
        else
            foreach ( $xml->link as $link )
                if ( $link['rel'] == 'http://schemas.google.com/g/2005#resumable-create-media' )
                    return $link['href'];
    }
    return false;
}

/**
 * Handle Google OAuth 2.0
 */
add_action('init', 'backup_auth');
function backup_auth() {
    if ( is_admin() && isset( $_GET['page'] ) && $_GET['page'] == 'backup' && isset( $_GET['action'] ) && $_GET['action'] == 'auth' ) {
        if ( isset( $_GET['state'] ) ) {
            if ( $_GET['state'] == 'token' )
                auth_token();
            else if ( $_GET['state'] == 'revoke' )
                auth_revoke();
        }
        else {
            $options = get_option( 'backup_options' );
            $options['client_id'] = $_POST['client_id'];
            $options['client_secret'] = $_POST['client_secret'];
            update_option( 'backup_options', $options );
            auth_request();
        }
    }
}

/**
 * Acquire single-use authorization code from Google OAuth 2.0
 */
function auth_request() {
    $options = get_option( 'backup_options' );
    $params = array(
        'response_type' => 'code',
        'client_id' => $options['client_id'],
        'redirect_uri' => admin_url('options-general.php?page=backup&action=auth'),
        'scope' => 'https://www.googleapis.com/auth/drive.file https://docs.google.com/feeds/ https://docs.googleusercontent.com/ https://spreadsheets.google.com/feeds/',
        'state' => 'token',
        'access_type' => 'offline',
    );
    header('Location: https://accounts.google.com/o/oauth2/auth?'.http_build_query($params));
}

/**
 * Get a Google account refresh token using the code received from auth_request
 */
function auth_token() {
    $options = get_option( 'backup_options' );
    if( isset( $_GET['code'] ) ) {
        $context = array(
            'http' => array(
                'method'  => 'POST',
                'header'  => 'Content-type: application/x-www-form-urlencoded',
                'content' => http_build_query( array(
                    'code' => $_GET['code'],
                    'client_id' => $options['client_id'],
                    'client_secret' => $options['client_secret'],
                    'redirect_uri' => admin_url('options-general.php?page=backup&action=auth'),
                    'grant_type' => 'authorization_code'
                ) )
            )
        );
        $result = @file_get_contents('https://accounts.google.com/o/oauth2/token', false, stream_context_create($context));
        if($result) {
            $result = json_decode( $result, true );
            if ( isset( $result['refresh_token'] ) ) {
                $options['token'] = $result['refresh_token']; // Save token
                update_option('backup_options', $options);
                header('Location: '.admin_url('options-general.php?page=backup&message=' . __( 'Authorization was successful.', 'backup' ) ) );
            }
            else
                header('Location: '.admin_url('options-general.php?page=backup&error=' . __( 'No refresh token was received!', 'backup' ) ) );
        }
        else
            header('Location: '.admin_url('options-general.php?page=backup&error=' . __( 'Bad response!', 'backup' ) ) );
    }
    else
        header('Location: '.admin_url('options-general.php?page=backup&error=' . __( 'Authrization failed!', 'backup' ) ) );
}

/**
 * Get a Google account access token using the refresh token
 */
function access_token( $token, $client_id, $client_secret ) {
    $context = array(
        'http' => array(
            'method'  => 'POST',
            'header'  => 'Content-type: application/x-www-form-urlencoded',
            'content' => http_build_query( array(
                'refresh_token' => $token,
                'client_id' => $client_id,
                'client_secret' => $client_secret,
                'grant_type' => 'refresh_token'
            ) )
        )
    );
    $result = @file_get_contents('https://accounts.google.com/o/oauth2/token', false, stream_context_create($context));
    if($result) {
        $result = json_decode( $result, true );
        if ( isset( $result['access_token'] ) )
            return $result['access_token'];
        else
            return false;
    }
    else
        return false;
}

/**
 * Revoke a Google account refresh token
 */
function auth_revoke() {
    $options = get_option( 'backup_options' );
    @file_get_contents( 'https://accounts.google.com/o/oauth2/revoke?token=' . $options['token'] );
    $options['token'] = '';
    update_option( 'backup_options', $options );
    header( 'Location: '.admin_url( 'options-general.php?page=backup&message=' . __( 'Authorization revoked.', 'backup' ) ) );
}

/**
 * Create a Zip archive from a file or directory
 * @param string $source File or Directory to be archived
 * @param string $destination Destination file where the archive will be stored
 * @param array $exclude Folder to exclude from archive, defaults to empty string
 */
function zip( $source, $destination, $exclude = '' ) { 
    if ( !file_exists( $source ) ) {
        backup_log( 'ERROR', 'Source file or directory to be archived does not exist.', __FILE__, __LINE__ );
        return false;
    }

    $zip = new ZipArchive();
    if ( $res = $zip->open( $destination, ZIPARCHIVE::CREATE ) != true ) {
        backup_log('ERROR', $res, __FILE__, __LINE__);
        return false;
    }

    $source = str_replace( '\\', '/', realpath( $source ) );

    if ( is_dir( $source ) === true ) {
        $files = directory_list( $source, $exclude, true );

        foreach ( $files as $file ) {
            $file = str_replace( '\\', '/', realpath( $file ) );
            if ( is_dir( $file ) === true ) {
                $zip->addEmptyDir( str_replace( $source . '/', '', $file . '/' ) );
            }
            else if ( is_file( $file ) === true ) {
                $zip->addFile( $file, str_replace( $source . '/', '', $file ) );
            } 
        }
    }
    else if (is_file($source) === true) {
        $zip->addFromString(basename($source), file_get_contents($source));
    }

    return $zip->close();
}

/**
 * directory_list
 *
 * return an array containing all files/directories at a file system path
 *
 * @param string $base_path Absolute or relative path to the base directory
 * @param string $exclude Pipe delimited string of files to always ignore
 * @param boolean $recursive Descend directory to the bottom?
 * @return array $result_list Array or false
 */
function directory_list($base_path, $exclude = "", $recursive = true) {
    $base_path = rtrim($base_path, "/") . "/";

    if ( !is_dir( $base_path ) ) {
        backup_log( 'ERROR', __FUNCTION__ . $base_path . " is not a directory.", __FILE__, __LINE__ );
        return false;
    }

    $exclude_files = array( ".", "..", ".DS_Store", ".svn" );
    $exclude_paths = explode( "|", $exclude );

    $result_list = array();

    if (!$folder_handle = opendir($base_path)) {
        backup_log( 'ERROR', __FUNCTION__ . "Could not open directory at: " . $base_path . ".", __FILE__, __LINE__ );
        return false;
    }
    else {
        while ( false !== ( $filename = readdir( $folder_handle ) ) ) {
            if ( !in_array( $filename, $exclude_files ) && !in_array( $base_path . $filename, $exclude_paths ) ) {
                if ( is_dir( $base_path . $filename . "/" ) ) {
                    $result_list[] = $base_path . $filename;
                    if( $recursive ) {
                        $temp_list = directory_list( $base_path . $filename . "/", $exclude, $recursive);
                        if ( ! empty( $temp_list ) )
                            foreach ( $temp_list as $item )
                                $result_list[] = $item;
                    }
                } else {
                    $result_list[] = $base_path . $filename;
                }
            }
        }
        closedir($folder_handle);
        
        return $result_list;
    }
}

/**
 * Dump the WordPress database to a sql file 
 *
 * @return boolean Returns TRUE on success, FALSE on failure
 */
function db_dump() {
    global $wpdb;
    $dump_file = WP_CONTENT_DIR . '/backup.sql';

    backup_log( 'NOTICE', 'Attempting to dump database to ' . $dump_file );
    $timer_start = microtime( true );

    $handle = fopen( $dump_file, 'wb' );

    if ( ! $handle )
        backup_log( 'ERROR', 'Could not open ' . $dump_file . ' for writing.', __FILE__, __LINE__ );
    else {

        fwrite( $handle, "/**\n" );
        fwrite( $handle, " * SQL Dump created with Backup for WordPress\n" );
        fwrite( $handle, " *\n" );
        fwrite( $handle, " * http://hel.io/wordpress/backup\n" );
        fwrite( $handle, " */\n\n" );

        fwrite( $handle, "/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;\n" );
        fwrite( $handle, "/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;\n" );
        fwrite( $handle, "/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;\n" );
        fwrite( $handle, "/*!40101 SET NAMES " . DB_CHARSET . " */;\n" );
        fwrite( $handle, "/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;\n" );
        fwrite( $handle, "/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;\n" );
        fwrite( $handle, "/*!40111 SET @OLD_SQL_NOTES=@@SQL_NOTES, SQL_NOTES=0 */;\n\n" );


        $tables = $wpdb->get_results( "SHOW TABLES", ARRAY_A );

        if ( empty( $tables ) )
            backup_log( 'ERROR', 'The query "SHOW TABLES" did not return any tables.', __FILE__, __LINE__ );
        else {
            foreach ( $tables as $table_array ) {
                $table = array_shift( array_values( $table_array ) );
                $create = $wpdb->get_var( "SHOW CREATE TABLE " . $table, 1 );

                backup_log( 'NOTICE', 'Dumping table `' . $table . '`' );
                
                fwrite( $handle, "/* Dump of table `" . $table . "`\n" );
                fwrite( $handle, " * ------------------------------------------------------------*/\n\n" );
                
                fwrite( $handle, "DROP TABLE IF EXISTS `" . $table . "`;\n\n" . $create . ";\n\n" );

                $data = $wpdb->get_results("SELECT * FROM `" . $table . "`", ARRAY_A );

                if ( ! empty( $data ) ) {
                    fwrite( $handle, "LOCK TABLES `" . $table . "` WRITE;\n" );
                    if ( strpos( $create, 'MyISAM' ) !== false )
                        fwrite( $handle, "/*!40000 ALTER TABLE `".$table."` DISABLE KEYS */;\n\n" );
                    foreach ( $data as $entry ) {
                        foreach ( $entry as $key => $value ) {
                            if ( $value === NULL )
                                $entry[$key] = "NULL";
                            elseif ( $value === "" || $value === false )
                                $entry[$key] = "''";
                            elseif ( !is_numeric( $value ) )
                                $entry[$key] = "'" . mysql_real_escape_string($value) . "'";
                        }
                        fwrite( $handle, "INSERT INTO `" . $table . "` ( " . implode( ", ", array_keys( $entry ) ) . " ) VALUES ( " . implode( ", ", $entry ) . " );\n" );
                    }
                    if ( strpos( $create, 'MyISAM' ) !== false )
                        fwrite( $handle, "\n/*!40000 ALTER TABLE `".$table."` ENABLE KEYS */;" );
                    fwrite( $handle, "\nUNLOCK TABLES;\n\n" );
                }
            }
        }    

        fwrite( $handle, "/*!40111 SET SQL_NOTES=@OLD_SQL_NOTES */;\n" );
        fwrite( $handle, "/*!40101 SET SQL_MODE=@OLD_SQL_MODE */;\n" );
        fwrite( $handle, "/*!40014 SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS */;\n" );
        fwrite( $handle, "/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;\n" );
        fwrite( $handle, "/*!40101 SET CHARACTER_SET_RESULTS=@OLD_CHARACTER_SET_RESULTS */;\n" );
        fwrite( $handle, "/*!40101 SET COLLATION_CONNECTION=@OLD_COLLATION_CONNECTION */;\n" );

        fclose( $handle );
        backup_log( 'NOTICE', 'Database dump completed in ' . ( microtime( true ) - $timer_start ) . ' seconds.' );
        return true;
    }
    return false;
}

/**
 * Recursively remove a directory
 *
 * @param string $dir The path to the directory to be removed
 */
function rrmdir( $dir ) {
    foreach ( glob( $dir . '/*' ) as $file ) {
        if ( is_dir( $file ) )
            rrmdir( $file );
        else
            unlink( $file );
    }
    rmdir( $dir );
}

/**
 * Custom logging function for the backup plugin
 *
 * @param  string $type    Type of message that we are logging. Should be 'NOTICE', 'WARNING' or 'ERROR'.
 * @param  string $message The message we are logging
 * @param  string $file    The file where the function was called from. The funciton should always be called with __FILE__ as the $file parameter.
 * @param  string $line    The line where the function was called from. The function should always be called with __LINE__ as the $line parameter.
 * @return boolean         Returns TRUE on success, FALSE on failure.
 */
function backup_log( $type, $message, $file = '', $line = '' ) {
    return error_log( date( "Y-m-d\tH:i:s" ) . "\t" . $type . "\t" . $message . "\t" . $file . "\t" . $line . "\n", 3, WP_CONTENT_DIR . '/backup/backup.log' );
}